@@ -22,14 +22,14 @@ class ConditionalOffer(models.Model):
     description = models.TextField(blank=True, null=True)
 
     # Offers come in a few different types:
-    # (a) Offers that are available to all customers on the site.  Eg a 
+    # (a) Offers that are available to all customers on the site.  Eg a
     #     3-for-2 offer.
     # (b) Offers that are linked to a voucher, and only become available once
     #     that voucher has been applied to the basket
     # (c) Offers that are linked to a user.  Eg, all students get 10% off.  The code
     #     to apply this offer needs to be coded
-    # (d) Session offers - these are temporarily available to a user after some trigger 
-    #     event.  Eg, users coming from some affiliate site get 10% off.     
+    # (d) Session offers - these are temporarily available to a user after some trigger
+    #     event.  Eg, users coming from some affiliate site get 10% off.
     TYPE_CHOICES = (
         (SITE, "Site offer - available to all users"),
         (VOUCHER, "Voucher offer - only available after entering the appropriate voucher code"),
@@ -42,7 +42,7 @@ class ConditionalOffer(models.Model):
     benefit = models.ForeignKey('offer.Benefit')
 
     # Range of availability.  Note that if this is a voucher offer, then these
-    # dates are ignored and only the dates from the voucher are used to determine 
+    # dates are ignored and only the dates from the voucher are used to determine
     # availability.
     start_date = models.DateField(blank=True, null=True)
     end_date = models.DateField(blank=True, null=True)
@@ -52,7 +52,7 @@ class ConditionalOffer(models.Model):
 
     # We track some information on usage
     total_discount = models.DecimalField(decimal_places=2, max_digits=12, default=Decimal('0.00'))
-    
+
     date_created = models.DateTimeField(auto_now_add=True)
 
     objects = models.Manager()
@@ -65,22 +65,22 @@ class ConditionalOffer(models.Model):
 
     class Meta:
         ordering = ['-priority']
-        
+
     def __unicode__(self):
-        return self.name    
+        return self.name
 
     def clean(self):
         if self.start_date and self.end_date and self.start_date > self.end_date:
             raise exceptions.ValidationError('End date should be later than start date')
-        
+
     def is_active(self, test_date=None):
         if not test_date:
             test_date = datetime.date.today()
         return self.start_date <= test_date and test_date < self.end_date
-    
+
     def is_condition_satisfied(self, basket):
         return self._proxy_condition().is_satisfied(basket)
-        
+
     def apply_benefit(self, basket):
         u"""
         Applies the benefit to the given basket and returns the discount.
@@ -88,18 +88,18 @@ class ConditionalOffer(models.Model):
         if not self.is_condition_satisfied(basket):
             return Decimal('0.00')
         return self._proxy_benefit().apply(basket, self._proxy_condition())
-        
+
     def set_voucher(self, voucher):
         self._voucher = voucher
-        
+
     def get_voucher(self):
-        return self._voucher        
-        
+        return self._voucher
+
     def _proxy_condition(self):
         u"""
         Returns the appropriate proxy model for the condition
         """
-        field_dict = self.condition.__dict__
+        field_dict = dict(self.condition.__dict__)
         if '_state' in field_dict:
             del field_dict['_state']
         if self.condition.type == self.condition.COUNT:
@@ -109,12 +109,12 @@ class ConditionalOffer(models.Model):
         elif self.condition.type == self.condition.COVERAGE:
             return CoverageCondition(**field_dict)
         return self.condition
-    
+
     def _proxy_benefit(self):
         u"""
         Returns the appropriate proxy model for the condition
         """
-        field_dict = self.benefit.__dict__
+        field_dict = dict(self.benefit.__dict__)
         if '_state' in field_dict:
             del field_dict['_state']
         if self.benefit.type == self.benefit.PERCENTAGE:
@@ -126,7 +126,7 @@ class ConditionalOffer(models.Model):
         elif self.benefit.type == self.benefit.FIXED_PRICE:
             return FixedPriceBenefit(**field_dict)
         return self.benefit
-        
+
 
 class Condition(models.Model):
     COUNT, VALUE, COVERAGE = ("Count", "Value", "Coverage")
@@ -145,10 +145,10 @@ class Condition(models.Model):
         elif self.type == self.COVERAGE:
             return u"Basket includes %d distinct products from %s" % (self.value, unicode(self.range).lower())
         return u"Basket includes %d value from %s" % (self.value, unicode(self.range).lower())
-    
+
     def consume_items(self, basket, lines=None):
         return ()
-    
+
     def is_satisfied(self, basket):
         """
         Determines whether a given basket meets this condition.  This is
@@ -156,7 +156,7 @@ class Condition(models.Model):
         responsible for implementing it correctly.
         """
         return False
-    
+
 
 class Benefit(models.Model):
     PERCENTAGE, FIXED, MULTIBUY, FIXED_PRICE = ("Percentage", "Absolute", "Multibuy", "Fixed price")
@@ -173,11 +173,11 @@ class Benefit(models.Model):
 
     price_field = 'price_incl_tax'
 
-    # If this is not set, then there is no upper limit on how many products 
+    # If this is not set, then there is no upper limit on how many products
     # can be discounted by this benefit.
     max_affected_items = models.PositiveIntegerField(blank=True, null=True, help_text="""Set this
         to prevent the discount consuming all items within the range that are in the basket.""")
-    
+
     def __unicode__(self):
         if self.type == self.PERCENTAGE:
             desc = u"%s%% discount on %s" % (self.value, unicode(self.range).lower())
@@ -192,10 +192,10 @@ class Benefit(models.Model):
         elif self.max_affected_items > 1:
             desc += u" (max %d items)" % self.max_affected_items
         return desc
-    
+
     def apply(self, basket, condition=None):
         return Decimal('0.00')
-    
+
     def clean(self):
         if self.value is None:
             if not self.type:
@@ -234,14 +234,14 @@ class Range(models.Model):
     excluded_products = models.ManyToManyField('catalogue.Product', related_name='excludes', blank=True)
     classes = models.ManyToManyField('catalogue.ProductClass', related_name='classes', blank=True)
     included_categories = models.ManyToManyField('catalogue.Category', related_name='includes', blank=True)
-    
+
     __included_product_ids = None
     __excluded_product_ids = None
     __class_ids = None
 
     def __unicode__(self):
-        return self.name    
-        
+        return self.name
+
     def contains_product(self, product):
         """
         Check whether the passed product is part of this range
@@ -258,33 +258,33 @@ class Range(models.Model):
         if self.includes_all_products:
             return True
         if product.product_class_id in self._class_ids():
-            return True   
+            return True
         included_product_ids = self._included_product_ids()
         if product.id in included_product_ids:
             return True
-        test_categories = self.included_categories.all() 
+        test_categories = self.included_categories.all()
         if test_categories:
             for category in product.categories.all():
                 for test_category in test_categories:
                     if category == test_category or category.is_descendant_of(test_category):
                         return True
         return False
-    
+
     def _included_product_ids(self):
         if None == self.__included_product_ids:
             self.__included_product_ids = [row['id'] for row in self.included_products.values('id')]
         return self.__included_product_ids
-    
+
     def _excluded_product_ids(self):
         if None == self.__excluded_product_ids:
             self.__excluded_product_ids = [row['id'] for row in self.excluded_products.values('id')]
         return self.__excluded_product_ids
-    
+
     def _class_ids(self):
         if None == self.__class_ids:
             self.__class_ids = [row['id'] for row in self.classes.values('id')]
         return self.__class_ids
-        
+
 
 class CountCondition(Condition):
     """
@@ -305,7 +305,7 @@ class CountCondition(Condition):
             if num_matches >= self.value:
                 return True
         return False
-    
+
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
@@ -323,8 +323,8 @@ class CountCondition(Condition):
             if len(consumed_products) == value:
                 break
         return consumed_products
-        
-        
+
+
 class CoverageCondition(Condition):
     """
     An offer condition dependent on the NUMBER of matching items from the basket.
@@ -346,7 +346,7 @@ class CoverageCondition(Condition):
             if len(covered_ids) >= self.value:
                 return True
         return False
-    
+
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
@@ -364,7 +364,7 @@ class CoverageCondition(Condition):
             if len(consumed_products) >= value:
                 break
         return consumed_products
-    
+
     def get_value_of_satisfying_items(self, basket):
         covered_ids = []
         value = Decimal('0.00')
@@ -375,8 +375,8 @@ class CoverageCondition(Condition):
             if len(covered_ids) >= self.value:
                 return value
         return value
-        
-        
+
+
 class ValueCondition(Condition):
     """
     An offer condition dependent on the VALUE of matching items from the basket.
@@ -397,12 +397,12 @@ class ValueCondition(Condition):
             if value_of_matches >= self.value:
                 return True
         return False
-    
+
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
         can't be reused in other offers.
-        
+
         We allow lines to be passed in as sometimes we want them sorted
         in a specific order.
         """
@@ -417,11 +417,11 @@ class ValueCondition(Condition):
                 if not price:
                     continue
                 quantity_to_consume = min(line.quantity_without_discount,
-                                          int(((value - value_of_matches)/price).quantize(Decimal(1),
+                                          int(((value - value_of_matches) / price).quantize(Decimal(1),
                                                                                               ROUND_UP)))
                 value_of_matches += price * quantity_to_consume
                 line.consume(quantity_to_consume)
-                consumed_products.extend((line.product,)*int(quantity_to_consume))
+                consumed_products.extend((line.product,) * int(quantity_to_consume))
             if value_of_matches >= value:
                 break
         return consumed_products
@@ -442,20 +442,20 @@ class PercentageDiscountBenefit(Benefit):
         discount = Decimal('0.00')
         affected_items = 0
         max_affected_items = self._effective_max_affected_items()
-        
+
         for line in basket.all_lines():
             if affected_items >= max_affected_items:
                 break
             product = line.product
             if self.range.contains_product(product) and product.has_stockrecord:
                 price = getattr(product.stockrecord, self.price_field)
-                quantity = min(line.quantity_without_discount, 
+                quantity = min(line.quantity_without_discount,
                                max_affected_items - affected_items)
-                line_discount = self.round(self.value/100 * price * int(quantity))
+                line_discount = self.round(self.value / 100 * price * int(quantity))
                 line.discount(line_discount, quantity)
                 affected_items += quantity
                 discount += line_discount
-                
+
         if discount > 0 and condition:
             condition.consume_items(basket)
         return discount
@@ -473,7 +473,7 @@ class AbsoluteDiscountBenefit(Benefit):
         discount = Decimal('0.00')
         affected_items = 0
         max_affected_items = self._effective_max_affected_items()
-        
+
         for line in basket.all_lines():
             if affected_items >= max_affected_items:
                 break
@@ -484,32 +484,33 @@ class AbsoluteDiscountBenefit(Benefit):
                     # Avoid zero price products
                     continue
                 remaining_discount = self.value - discount
-                quantity_affected = int(min(line.quantity_without_discount, 
+                quantity_affected = int(min(line.quantity_without_discount,
                                         max_affected_items - affected_items,
                                         math.ceil(remaining_discount / price)))
-                
+
                 # Update line with discounts
                 line_discount = self.round(min(remaining_discount, quantity_affected * price))
-                line.discount(line_discount, quantity_affected)
-                
+                if not condition:
+                    line.discount(line_discount, quantity_affected)
+
                 # Update loop vars
                 affected_items += quantity_affected
                 remaining_discount -= line_discount
                 discount += line_discount
         if discount > 0 and condition:
             condition.consume_items(basket)
-            
+
         return discount
 
 
 class FixedPriceBenefit(Benefit):
     """
-    An offer benefit that gives the items in the condition for a 
+    An offer benefit that gives the items in the condition for a
     fixed price.  This is useful for "bundle" offers.
-    
+
     Note that we ignore the benefit range here and only give a fixed price
     for the products in the condition range.
-    
+
     The condition should be a count condition
     """
     class Meta:
@@ -523,22 +524,22 @@ class FixedPriceBenefit(Benefit):
         for line in basket.all_lines():
             product = line.product
             if condition.range.contains_product(product) and line.quantity_without_discount > 0:
-                # Line is available - determine quantity to consume and 
+                # Line is available - determine quantity to consume and
                 # record the total of the consumed products
                 if isinstance(condition, CoverageCondition):
                     quantity = 1
                 else:
                     quantity = min(line.quantity_without_discount, num_permitted - num_covered)
                 num_covered += quantity
-                product_total += quantity*line.unit_price_incl_tax
+                product_total += quantity * line.unit_price_incl_tax
                 covered_lines.append((line, quantity))
             if num_covered >= num_permitted:
                 break
         discount = max(product_total - self.value, Decimal('0.00'))
-        
+
         if not discount:
             return discount
-        
+
         # Apply discount weighted by original value of line
         discount_applied = Decimal('0.00')
         last_line = covered_lines[-1][0]
@@ -551,13 +552,13 @@ class FixedPriceBenefit(Benefit):
                 line_discount = self.round(discount * (line.unit_price_incl_tax * quantity) / product_total)
             line.discount(line_discount, quantity)
             discount_applied += line_discount
-        return discount 
-        
+        return discount
+
         # Apply discount weighted by original value of line
         for line, quantity in covered_lines:
-            line_discount = self.round(discount * (line.unit_price_incl_tax * quantity) / product_total)  
+            line_discount = self.round(discount * (line.unit_price_incl_tax * quantity) / product_total)
             line.discount(line_discount.quantize(Decimal('.01')), quantity)
-        return discount 
+        return discount
 
 
 class MultibuyDiscountBenefit(Benefit):
